<?php
/**
 * Copyright 2015 Klarna AB
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * File containing the Klarna_Checkout_Resource interface
 *
 * PHP version 5.3
 *
 * @category  Payment
 * @package   Klarna_Checkout
 * @author    Klarna <support@klarna.com>
 * @copyright 2015 Klarna AB
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache license v2.0
 * @link      http://developers.klarna.com/
 */

/**
 * Interface for the resource object
 *
 * @category  Payment
 * @package   Klarna_Checkout
 * @author    Majid G. <majid.garmaroudi@klarna.com>
 * @author    David K. <david.keijser@klarna.com>
 * @copyright 2015 Klarna AB
 * @license   http://www.apache.org/licenses/LICENSE-2.0 Apache license v2.0
 * @link      http://developers.klarna.com/
 */
interface Klarna_Checkout_ResourceInterface
{
    /**
     * Get the URL of the resource
     *
     * @return string
     */
    public function getLocation();

    /**
     * Set the URL of the resource
     *
     * @param string $location URL of the resource
     *
     * @return void
     */
    public function setLocation($location);

    /**
     * Return content type of the resource
     *
     * @return string Content type
     */
    public function getContentType();

    /**
     * Set the content type
     *
     * @param string $contentType Content type
     *
     * @return void
     */
    public function setContentType($contentType);

    /**
     * Return accept header of the resource
     *
     * @return string Accept header
     */
    public function getAccept();

    /**
     * Set the accept type
     *
     * @param string $accept Accept type
     *
     * @return void
     */
    public function setAccept($accept);

    /**
     * Update resource with the new data
     *
     * @param array $data data
     *
     * @return void
     */
    public function parse(array $data);

    /**
     * Basic representation of the object
     *
     * @return array data
     */
    public function marshal();
}
